#ifndef VIEWERWINDOW_H
#define VIEWERWINDOW_H

#include <fx.h>
#include "FXOSGCompositeViewer.h"

using namespace FX;
using namespace osgFOX;

class ViewerFoldingList;

class ViewerWindow : public FXMainWindow
{
FXDECLARE(ViewerWindow)
protected:
  // Identifiers for the property sheet switcher's panels
  enum
  {
    NODE_PANEL = 0,
    SWITCH_PANEL = 1,
    TRANSFORM_PANEL = 2,
    GEOMETRY_PANEL = 3,
    LAST_PANEL = 4
  };
protected:
  FXbool dirty_;                                // The current scene has been modified
  FXbool drawingViewport_;                      // View creation mode is active, for drawing a view with mouse
  FXint drawStart_[2];                          // (x, y) coordinate for start of viewport, stored on mouse press
  FXString loadScenePath_;
  FXString currentFile_;
  FXToolBar* toolbar_;
  FXStatusBar* statusbar_;
  FXMenuPane* filemenu_;
  FXMenuPane* viewmenu_;
  ViewerFoldingList* list_;
  FXSwitcher* switcher_;
  FXTextField* nodenames_[LAST_PANEL];
  FXList* switchlist_;
  FXTextField* transforms_[10];
  FXTable* geomtable_;
  FXGLVisual* glvisual_;
  FXOSGCompositeViewer* viewer_;
  osg::ref_ptr<osg::Node> root_;
  FXFoldingItem* currentItem_;                  // The current node selected from the folding list
  FXTableRange vertexRange_;                    // Range of vertex cells modified in table, requires delayed processing (chore)
protected:
  ViewerWindow() { }
  void readRegistry();
  void writeRegistry();
  bool isAncestor(osg::Group* target, osg::Node* node);      // Tests if target is an ancestor of node
  void removeAllParents(osg::Node* node);
  void setSceneRoot(osg::Node* node);
  void resetCameras();
  void beginCreateViewport();
  void endCreateViewport();
  FXint fixCreateViewportX(FXint x);
  FXint fixCreateViewportY(FXint y);
  bool open();
  bool saveAs();
  FXString makeNodeLabel(osg::Node* node);
  FXString makeDrawableLabel(osg::Drawable* item);
  FXString makePrimitiveSetLabel(osg::PrimitiveSet* set);
  void populateList();
  void process(osg::Node* node, FXFoldingItem* parent);
  void processGeode(osg::Geode* geode, FXFoldingItem* parent);
public:
  enum
  {
    ID_QUIT = FXMainWindow::ID_LAST,
    ID_TITLEBAR,
    ID_LOADSCENE,
    ID_SAVESCENE,
    ID_SAVESCENEAS,
    ID_MANAGEVIEWPORTS,
    ID_DRAWVIEWPORT,
    ID_RESETCAMERAS,
    ID_SETBACKCOLOR,
    ID_SETAMBIENTCOLOR,
    ID_CANVAS,
    ID_LIST,

    // Node property panel input
    ID_NODENAME,
    ID_SWITCHNODE,
    ID_TRANSLATION_X,
    ID_TRANSLATION_Y,
    ID_TRANSLATION_Z,
    ID_ROTATION_A,
    ID_ROTATION_X,
    ID_ROTATION_Y,
    ID_ROTATION_Z,
    ID_SCALE_X,
    ID_SCALE_Y,
    ID_SCALE_Z,
    ID_GEOMTABLE,
    ID_GEOMCHORE,

    ID_DRAWCHORE,
    ID_DND,
    ID_LAST
  };
public:
  long onCmdQuit(FXObject*, FXSelector, void*);
  long onUpdTitleBar(FXObject*, FXSelector, void*);
  long onCmdLoadScene(FXObject*, FXSelector, void*);
  long onCmdSaveScene(FXObject*, FXSelector, void*);
  long onUpdSaveScene(FXObject*, FXSelector, void*);
  long onCmdSaveSceneAs(FXObject*, FXSelector, void*);
  long onCmdManageViewports(FXObject*, FXSelector, void*);
  long onCmdDrawViewport(FXObject*, FXSelector, void*);
  long onCmdResetCameras(FXObject*, FXSelector, void*);
  long onCmdSetBackColor(FXObject*, FXSelector, void*);
  long onCmdSetAmbientColor(FXObject*, FXSelector, void*);
  long onLeftBtnPress(FXObject*, FXSelector, void*);
  long onLeftBtnRelease(FXObject*, FXSelector, void*);
  long onMotion(FXObject*, FXSelector, void*);
  long onCmdList(FXObject*, FXSelector, void*);
  long onCmdNodeName(FXObject*, FXSelector, void*);
  long onCmdSwitchNode(FXObject*, FXSelector, void*);
  long onCmdPositionAttitudeTransform(FXObject*, FXSelector, void*);
  long onCmdGeometryTable(FXObject*, FXSelector, void*);
  long onGeometryChore(FXObject*, FXSelector, void*);
  long onDrawChore(FXObject*, FXSelector, void*);
  long onDNDDrop(FXObject*,FXSelector,void*);
  long onDNDRequest(FXObject*,FXSelector,void*);
public:
  ViewerWindow(FXApp* app);

  virtual void create();

  FXuint getViewBorderWidth() const { return viewer_->getViewBorderWidth(); }

  void setViewBorderWidth(FXuint width) { viewer_->setViewBorderWidth(width); }

  FXColor getViewBorderColor() const { return viewer_->getViewBorderColor(); }

  void setViewBorderColor(FXColor color) { viewer_->setViewBorderColor(color); }

  FXuint getFocusViewBorderWidth() const { return viewer_->getFocusViewBorderWidth(); }

  void setFocusViewBorderWidth(FXuint width) { viewer_->setFocusViewBorderWidth(width); }

  FXColor getFocusViewBorderColor() const { return viewer_->getFocusViewBorderColor(); }

  void setFocusViewBorderColor(FXColor color) { viewer_->setFocusViewBorderColor(color); }

  void addView(const std::string& name, bool fixed, double x, double y, double w, double h);

  void removeView(unsigned int index);

  bool loadModel(const char* name);

  bool saveModel(const char* name);

  virtual ~ViewerWindow();
};

#endif
