#include "RatioViewport.h"
#include "ViewerWindow.h"
#include "ViewportDialog.h"

FXDEFMAP(ViewportDialog) ViewportDialogMap[] = {
  FXMAPFUNC(SEL_SELECTED, ViewportDialog::ID_LIST, ViewportDialog::onCmdList),
  FXMAPFUNC(SEL_COMMAND, ViewportDialog::ID_ADD, ViewportDialog::onCmdAdd),
  FXMAPFUNC(SEL_COMMAND, ViewportDialog::ID_REMOVE, ViewportDialog::onCmdRemove),
  FXMAPFUNC(SEL_UPDATE, ViewportDialog::ID_REMOVE, ViewportDialog::onUpdRemove),
  FXMAPFUNC(SEL_COMMAND, ViewportDialog::ID_NAME, ViewportDialog::onCmdName),
  FXMAPFUNCS(SEL_COMMAND, ViewportDialog::ID_DIM_X, ViewportDialog::ID_DIM_HEIGHT, ViewportDialog::onCmdDimensions),
  FXMAPFUNC(SEL_COMMAND, ViewportDialog::ID_OK, ViewportDialog::onCmdOK),
  FXMAPFUNC(SEL_COMMAND, ViewportDialog::ID_APPLY, ViewportDialog::onCmdApply),
  FXMAPFUNC(SEL_UPDATE, ViewportDialog::ID_APPLY, ViewportDialog::onUpdApply)
};

FXIMPLEMENT(ViewportDialog, FXDialogBox, ViewportDialogMap, ARRAYNUMBER(ViewportDialogMap))


ViewportDialog::ViewportDialog(FXWindow* parent, ViewerWindow* window, FXOSGCompositeViewer* viewer)
: FXDialogBox(parent, "Viewport Manager", DECOR_TITLE|DECOR_BORDER|DECOR_CLOSE|DECOR_RESIZE, 0,0,400,300, 0,0,0,0),
  dirty_(false),
  selectedItem_(0),
  window_(window),
  viewer_(viewer)
{
  setUp();
}

ViewportDialog::ViewportDialog(FXApp* app, ViewerWindow* window, FXOSGCompositeViewer* viewer)
: FXDialogBox(app, "Viewport Manager", DECOR_TITLE|DECOR_BORDER|DECOR_CLOSE|DECOR_RESIZE, 0,0,400,300, 0,0,0,0),
  dirty_(false),
  selectedItem_(0),
  window_(window),
  viewer_(viewer)
{
  setUp();
}

void ViewportDialog::create()
{
  FXDialogBox::create();
  scanViewports();
}

void ViewportDialog::setUp()
{
  FXVerticalFrame* contents = new FXVerticalFrame(this, LAYOUT_FILL);
  FXSplitter* manager = new FXSplitter(contents, SPLITTER_HORIZONTAL|LAYOUT_FILL);
  FXVerticalFrame* viewportframe = new FXVerticalFrame(manager, LAYOUT_FILL, 0,0,0,0, 0,0,0,0);
  FXVerticalFrame* listframe = new FXVerticalFrame(viewportframe, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL, 0,0,0,0, 0,0,0,0, 0,0);
  new FXLabel(listframe, "Viewports", NULL, FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X);
  list_ = new FXList(listframe, this, ID_LIST, LIST_BROWSESELECT|LAYOUT_FILL);

  FXHorizontalFrame* viewportbuttons = new FXHorizontalFrame(viewportframe, PACK_UNIFORM_WIDTH|LAYOUT_FILL_X);
  new FXButton(viewportbuttons, "Add", NULL, this, ID_ADD, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X, 0,0,0,0, 8,8);
  new FXButton(viewportbuttons, "Remove", NULL, this, ID_REMOVE, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X, 0,0,0,0, 8,8);

  FXVerticalFrame* groupframe = new FXVerticalFrame(manager, LAYOUT_FILL, 0,0,0,0, 0,0,0,0);
  FXGroupBox* elementbox = new FXGroupBox(groupframe, "Viewport Attributes", GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL, 0,0,0,0, 0,0,0,0, 0,0);
  FXHorizontalFrame* elementframe = new FXHorizontalFrame(elementbox, LAYOUT_FILL_X);
  new FXLabel(elementframe, "Name:", NULL, LAYOUT_CENTER_Y);
  fields_[NAME] = new FXTextField(elementframe, 0, this, ID_NAME, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y);

  FXMatrix* elementmat = new FXMatrix(elementbox, 4, MATRIX_BY_COLUMNS|LAYOUT_FILL);
  new FXLabel(elementmat, "X:", NULL, LAYOUT_CENTER_Y);
  fields_[X] = new FXTextField(elementmat, 0, this, ID_DIM_X, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y);
  new FXLabel(elementmat, "Y:", NULL, LAYOUT_CENTER_Y);
  fields_[Y] = new FXTextField(elementmat, 0, this, ID_DIM_Y, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y);
  new FXLabel(elementmat, "Width:", NULL, LAYOUT_CENTER_Y);
  fields_[WIDTH] = new FXTextField(elementmat, 0, this, ID_DIM_WIDTH, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y);
  new FXLabel(elementmat, "Height:", NULL, LAYOUT_CENTER_Y);
  fields_[HEIGHT] = new FXTextField(elementmat, 0, this, ID_DIM_HEIGHT, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|LAYOUT_CENTER_Y);

  new FXHorizontalSeparator(contents, SEPARATOR_GROOVE|LAYOUT_FILL_X);

  FXHorizontalFrame* buttons = new FXHorizontalFrame(contents, PACK_UNIFORM_WIDTH|LAYOUT_FILL_X);
  new FXButton(buttons, "Apply", NULL, this, ID_APPLY, FRAME_THICK|FRAME_SUNKEN|LAYOUT_RIGHT, 0,0,0,0, 8,8);
  new FXButton(buttons, "Cancel", NULL, this, FXDialogBox::ID_CANCEL, FRAME_THICK|FRAME_SUNKEN|LAYOUT_RIGHT, 0,0,0,0, 8,8);
  new FXButton(buttons, "Accept", NULL, this, ID_OK, FRAME_THICK|FRAME_SUNKEN|LAYOUT_RIGHT, 0,0,0,0, 8,8);
}

void ViewportDialog::scanViewports()
{
  if(viewer_ != NULL)
  {
    AttribList_t::iterator iter;
    for(iter = viewports_.begin(); iter != viewports_.end(); ++iter)
    {
      delete (*iter);
    }

    viewports_.clear();
    list_->clearItems();

    int width = viewer_->getWidth(), height = viewer_->getHeight();
    unsigned int counter = 0;
    unsigned int i, n = viewer_->getNumViews();
    for(i = 0; i < n; i++)
    {
      Attributes* a =  new Attributes();
      osg::Viewport* view = viewer_->getView(i)->getCamera()->getViewport();

      a->position = i;
      a->remove = false;
      a->name = viewer_->getView(i)->getName().c_str();
      if(a->name.empty())
      {
        a->name = (i == 0) ? "Main View" : FXStringFormat("View %d", ++counter);
      }

      RatioViewport* rview = dynamic_cast<RatioViewport*>(view);
      if(rview != NULL)
      {
        a->x = rview->rx() * width;
        a->y = rview->ry() * height;
        a->width = rview->rwidth() * width;
        a->height = rview->rheight() * height;
        a->fixed = false;
      }
      else
      {
        a->x = view->x();
        a->y = view->y();
        a->width = view->width();
        a->height = view->height();
        a->fixed = true;
      }

      // Add viewports to the container and the UI list
      viewports_.push_back(a);
      list_->appendItem(a->name, NULL, a, true);
    }

    // Re-select the current item
	if(selectedItem_ >= list_->getNumItems())
	{
	  selectedItem_ = list_->getNumItems() - 1;
	}

    list_->selectItem(selectedItem_, TRUE);
  }
}

long ViewportDialog::onCmdList(FXObject*, FXSelector, void* ptr)
{
  FXint index = (FXint)(FXival)ptr;
  Attributes* a = static_cast<Attributes*>(list_->getItemData(index));

  fields_[NAME]->setText(a->name);
  fields_[X]->setText(FXStringVal(a->x));
  fields_[Y]->setText(FXStringVal(a->y));
  fields_[WIDTH]->setText(FXStringVal(a->width));
  fields_[HEIGHT]->setText(FXStringVal(a->height));

  selectedItem_ = index;

  return 1;
}

long ViewportDialog::onCmdAdd(FXObject*, FXSelector, void*)
{
  Attributes* a =  new Attributes();

  a->name = FXStringFormat("NewViewport");
  a->position = -1;
  a->remove = false;
  a->x = 10;
  a->y = 10;
  a->width = 100;
  a->height = 100;
  a->fixed = false;

  // Add viewports to the container and the UI list
  viewports_.push_back(a);
  list_->appendItem(a->name, NULL, a);

  // Select the new view
  list_->selectItem(list_->getNumItems() - 1, true);

  dirty_ = true;

  return 1;
}

long ViewportDialog::onCmdRemove(FXObject*, FXSelector, void*)
{
  Attributes* a = static_cast<Attributes*>(list_->getItemData(selectedItem_));
  a->remove = true;

  list_->removeItem(selectedItem_);

  // Removal occurs only when selecetedItem_ is greater than 0
  selectedItem_--;

  list_->selectItem(selectedItem_, TRUE);

  dirty_ = true;

  return 1;
}

long ViewportDialog::onUpdRemove(FXObject* sender, FXSelector, void*)
{
  sender->handle(this, FXSEL(SEL_COMMAND, (selectedItem_ == 0) ? ID_DISABLE : ID_ENABLE), 0);
  return 1;
}

long ViewportDialog::onCmdName(FXObject*, FXSelector, void*)
{
  Attributes* a = static_cast<Attributes*>(list_->getItemData(selectedItem_));
  a->name = fields_[NAME]->getText();

  list_->setItemText(selectedItem_, a->name);

  dirty_ = true;

  return 1;
}

long ViewportDialog::onCmdDimensions(FXObject* obj, FXSelector sel, void*)
{
  Attributes* a = static_cast<Attributes*>(list_->getItemData(selectedItem_));
  switch(FXSELID(sel))
  {
    case ID_DIM_X:
      a->x = FXDoubleVal(fields_[X]->getText());
      break;
    case ID_DIM_Y:
      a->y = FXDoubleVal(fields_[Y]->getText());
      break;
    case ID_DIM_WIDTH:
      a->width = FXDoubleVal(fields_[WIDTH]->getText());
      break;
    case ID_DIM_HEIGHT:
      a->height = FXDoubleVal(fields_[HEIGHT]->getText());
      break;
  }

  dirty_ = true;

  return 1;
}

long ViewportDialog::onCmdOK(FXObject*, FXSelector, void*)
{
  handle(this, FXSEL(SEL_COMMAND, ID_APPLY), 0);
  handle(this, FXSEL(SEL_COMMAND, FXDialogBox::ID_ACCEPT), 0);
  return 1;
}

long ViewportDialog::onCmdApply(FXObject*, FXSelector, void*)
{
  unsigned int index = 0;
  AttribList_t::iterator iter;
  for(iter = viewports_.begin(); iter != viewports_.end(); ++iter)
  {
    Attributes* a = (*iter);

    // Update or remove
    if(a->remove)
    {
      window_->removeView(a->position);
    }
    else if(a->position == -1)
    {
      int width = viewer_->getWidth(), height = viewer_->getHeight();
      window_->addView(a->name.text(), a->fixed, a->x / width, a->y / height, a->width / width, a->height / height);
    }
    else
    {
      int width = viewer_->getWidth(), height = viewer_->getHeight();
      osg::Viewport* view = viewer_->getView(index)->getCamera()->getViewport();
      RatioViewport* rview = dynamic_cast<RatioViewport*>(view);
      osg::Camera* camera = viewer_->getView(index)->getCamera();
      if(rview != NULL)
      {
        if(a->fixed)
        {
          camera->setViewport(new osg::Viewport(a->x, a->y, a->width, a->height));
          camera->setProjectionMatrixAsPerspective(45.0, camera->getViewport()->width()/camera->getViewport()->height(), 0.1, 1000.0);
        }
        else
        {
          rview->setViewportRatio(a->x / width, a->y / height, a->width / width, a->height / height);
          rview->resize(width, height);
          camera->setProjectionMatrixAsPerspective(45.0, camera->getViewport()->width()/camera->getViewport()->height(), 0.1, 1000.0);
        }
      }
      else
      {
        if(a->fixed)
        {
          view->setViewport(a->x, a->y, a->width, a->height);
          camera->setProjectionMatrixAsPerspective(45.0, camera->getViewport()->width()/camera->getViewport()->height(), 0.1, 1000.0);
        }
        else
        {
          camera->setViewport(new RatioViewport(a->x / width, a->y / height, a->width / width, a->height / height, width, height));
          camera->setProjectionMatrixAsPerspective(45.0, camera->getViewport()->width()/camera->getViewport()->height(), 0.1, 1000.0);
        }
      }

      viewer_->getView(index)->setName(a->name.text());

      // Clear the background to ensure that the old scene is not visisble (viewport made smaller or moved; no longer covers full screen)
      ((osgViewer::GraphicsWindow*)viewer_)->makeCurrent();
      glClearColor(0.0, 0.0, 0.0, 0.0);
      glClear(GL_COLOR_BUFFER_BIT);
      ((osgViewer::GraphicsWindow*)viewer_)->swapBuffers();
      ((osgViewer::GraphicsWindow*)viewer_)->releaseContext();

      index++;
    }
  }

  dirty_ = false;

  scanViewports();

  return 1;
}

long ViewportDialog::onUpdApply(FXObject* sender, FXSelector, void*)
{
  sender->handle(this, FXSEL(SEL_COMMAND, (dirty_) ? ID_ENABLE : ID_DISABLE), 0);
  return 1;
}

ViewportDialog::~ViewportDialog()
{
  AttribList_t::iterator iter;
  for(iter = viewports_.begin(); iter != viewports_.end(); ++iter)
  {
    delete (*iter);
  }
}
