#ifndef VIEWPORTDIALOG_H
#define VIEWPORTDIALOG_H

#include <fx.h>
#include "FXOSGCompositeViewer.h"

using namespace FX;
using namespace osgFOX;

class ViewerWindow;

class ViewportDialog : public FXDialogBox
{
FXDECLARE(ViewportDialog)
protected:
  // Enumeration for text field array
  enum
  {
    NAME = 0,
    X = 1,
    Y = 2,
    WIDTH = 3,
    HEIGHT = 4
  };
protected:
  struct Attributes
  {
    double x;             // X position
    double y;             // Y position
    double width;         // Viewort width
    double height;        // Viewport height
    int position;         // Position in the composite viewer's viewport list, -1 is a new viewport to add
    bool fixed;           // Is the viewport dimension a fixed size or a ratio
    bool remove;          // The viewport is scheduled for deletion
    FXString name;        // Viewport name
  };
  typedef std::vector<Attributes*> AttribList_t;
protected:
  FXbool dirty_;                   // A viewport has been modified
  FXint selectedItem_;             // Current item selected in UI viewport list
  AttribList_t viewports_;         // Attributes for all viewports that are in the UI list and have been deleted
  ViewerWindow* window_;           // The application window containing the viewer
  FXOSGCompositeViewer* viewer_;   // The viewer with viewports to modify
  FXList* list_;                   // Viewport list
  FXTextField* fields_[5];         // Text fields with viewport attributes
protected:
  ViewportDialog() { }
  void setUp();
  void scanViewports();
public:
  enum
  {
    ID_LIST = FXDialogBox::ID_LAST,
    ID_ADD,
    ID_REMOVE,
    ID_NAME,
    ID_DIM_X,
    ID_DIM_Y,
    ID_DIM_WIDTH,
    ID_DIM_HEIGHT,
    ID_OK,
    ID_APPLY,
    ID_LAST
  };
public:
  long onCmdList(FXObject*, FXSelector, void*);
  long onCmdAdd(FXObject*, FXSelector, void*);
  long onCmdRemove(FXObject*, FXSelector, void*);
  long onUpdRemove(FXObject*, FXSelector, void*);
  long onCmdName(FXObject*, FXSelector, void*);
  long onCmdDimensions(FXObject*, FXSelector, void*);
  long onCmdOK(FXObject*, FXSelector, void*);
  long onCmdApply(FXObject*, FXSelector, void*);
  long onUpdApply(FXObject*, FXSelector, void*);
public:
  ViewportDialog(FXWindow* parent, ViewerWindow* window, FXOSGCompositeViewer* viewer);

  ViewportDialog(FXApp* app, ViewerWindow* window, FXOSGCompositeViewer* viewer);

  virtual void create();

  ~ViewportDialog();
};

#endif
