#ifndef FXOSGCOMPOSITEVIEWER_H
#define FXOSGCOMPOSITEVIEWER_H

#include <map>
#include <fx.h>
#include <fx3d.h>
#include <osgViewer/CompositeViewer>

#include "GraphicsWindowFOX.h"

using namespace FX;

namespace osgFOX
{

class FXOSGCompositeViewer : public GraphicsWindowFOX, public osgViewer::CompositeViewer
{
  FXDECLARE(FXOSGCompositeViewer)
protected:
  FXuint borderWidth_;         // Width for the viewport borders - NOTE: this should probably be a viewport attribute
  FXColor borderColor_;        // Color for the viewport borders - NOTE: this should probably be a viewport attribute
  FXuint focusWidth_;          // Width for the border of the focused viewport
  FXColor focusColor_;         // Color for the border of the focused viewport
protected:
  FXOSGCompositeViewer() { }
public:
  enum
  {
    ID_CANVAS = FXMainWindow::ID_LAST,
    ID_LAST
  };
public:
  long onConfigure(FXObject *,FXSelector,void *data);
  long onPaint(FXObject*, FXSelector, void* data);
public:
  ///Constructor for canvas with a private display list.
  FXOSGCompositeViewer(FXComposite *p,FXGLVisual *vis,FXObject *tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  ///Constructor for canvas with a shared display list.
  FXOSGCompositeViewer(FXComposite *p,FXGLVisual *vis,FXGLCanvas *sharegroup,FXObject *tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  ///Virtual destructor.
  virtual ~FXOSGCompositeViewer();

  ///Standard FOX GUI create function.
  virtual void create();

  FXuint getViewBorderWidth() const { return borderWidth_; }

  void setViewBorderWidth(FXuint width) { borderWidth_ = width; }

  FXColor getViewBorderColor() const { return borderColor_; }

  void setViewBorderColor(FXColor color) { borderColor_ = color; }

  FXuint getFocusViewBorderWidth() const { return focusWidth_; }

  void setFocusViewBorderWidth(FXuint width) { focusWidth_ = width; }

  FXColor getFocusViewBorderColor() const { return focusColor_; }

  void setFocusViewBorderColor(FXColor color) { focusColor_ = color; }

  void setUpBorderRenderingSupport();

  void drawViewportBorder(osgViewer::View* view);

  void draw();

  void removeView(osgViewer::View* view);

  osgUtil::SceneView* getSceneView(osgViewer::View* view) { return _cameraSceneViewMap[view->getCamera()].get(); }

  osgUtil::SceneView* getSceneView(unsigned view) { return _cameraSceneViewMap[_views[view]->getCamera()].get(); }
};

}

#endif // FXOSGCOMPOSITEVIEWER_H
