#ifndef GRAPHICSWINDOWFOX_H
#define GRAPHICSWINDOWFOX_H

#include <fx.h>
#include <fx3d.h>
#include <osgViewer/GraphicsWindow>

using namespace FX;

namespace osgFOX
{

class GraphicsWindowFOX : public FXGLCanvas, public osgViewer::GraphicsWindow
{
  FXDECLARE(GraphicsWindowFOX)
protected:
  bool valid_;
protected:
  GraphicsWindowFOX() { }
public:
  long onConfigure(FXObject *,FXSelector,void *);
  long onMotion(FXObject *,FXSelector,void *);
  long onMouseWheel(FXObject *,FXSelector,void *);
  long onKeyPress(FXObject *,FXSelector,void *);
  long onKeyRelease(FXObject *,FXSelector,void *);
  long onLeftBtnPress(FXObject *,FXSelector,void *);
  long onLeftBtnRelease(FXObject *,FXSelector,void *);
  long onMiddleBtnPress(FXObject *,FXSelector,void *);
  long onMiddleBtnRelease(FXObject *,FXSelector,void *);
  long onRightBtnPress(FXObject *,FXSelector,void *);
  long onRightBtnRelease(FXObject *,FXSelector,void *);
public:
  ///Constructor for canvas with a private display list.
  GraphicsWindowFOX(FXComposite *p,FXGLVisual *vis,FXObject *tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  ///Constructor for canvas with a shared display list.
  GraphicsWindowFOX(FXComposite *p,FXGLVisual *vis,FXGLCanvas *sharegroup,FXObject *tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  ///Virtual destructor.
  virtual ~GraphicsWindowFOX();

  ///Standard FOX GUI create function.
  virtual void create();
public:
  /// Return whether a valid and usable GraphicsContext has been created.
  virtual bool valid() const { return valid_; }

  /// Realise the GraphicsContext implementation,
  virtual bool realizeImplementation();

  /// Return true if the graphics context has been realised, and is ready to use, implementation.
  virtual bool isRealizedImplementation() const { return valid_; }

  /// Close the graphics context implementation.
  virtual void closeImplementation()  { valid_ = false; }

  /// Make this graphics context current.
  virtual bool makeCurrentImplementation() { return FXGLCanvas::makeCurrent(); }

  /// Release the graphics context.
  virtual bool releaseContextImplementation() { return FXGLCanvas::makeNonCurrent(); }

  /// Swap the front and back buffers.
  virtual void swapBuffersImplementation() { FXGLCanvas::swapBuffers(); }

  /// Override from GUIActionAdapter
  virtual void requestRedraw() { update(); }

  /// Override from GUIActionAdapter
  virtual void requestContinuousUpdate(bool needed=true) { }

  /// Override from GUIActionAdapter
  virtual void requestWarpPointer(float x, float y) { setCursorPosition((FXint)x, (FXint)y); }
};

}

#endif
