#include "GraphicsWindowFOX.h"

using namespace osgFOX;

FXDEFMAP(GraphicsWindowFOX) GraphicsWindowFOXMap[]={
  FXMAPFUNC(SEL_CONFIGURE,0,GraphicsWindowFOX::onConfigure),
  FXMAPFUNC(SEL_MOTION,0,GraphicsWindowFOX::onMotion),
  FXMAPFUNC(SEL_MOUSEWHEEL,0,GraphicsWindowFOX::onMouseWheel),
  FXMAPFUNC(SEL_KEYPRESS,0,GraphicsWindowFOX::onKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE,0,GraphicsWindowFOX::onKeyRelease),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,GraphicsWindowFOX::onLeftBtnPress),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,GraphicsWindowFOX::onLeftBtnRelease),
  FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,0,GraphicsWindowFOX::onMiddleBtnPress),
  FXMAPFUNC(SEL_MIDDLEBUTTONRELEASE,0,GraphicsWindowFOX::onMiddleBtnRelease),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS,0,GraphicsWindowFOX::onRightBtnPress),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,0,GraphicsWindowFOX::onRightBtnRelease)
};

FXIMPLEMENT(GraphicsWindowFOX,FXGLCanvas,GraphicsWindowFOXMap,ARRAYNUMBER(GraphicsWindowFOXMap))

//Private display list.
GraphicsWindowFOX::GraphicsWindowFOX(FXComposite* p,FXGLVisual *vis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h)
: FXGLCanvas(p,vis,tgt,sel,opts,x,y,w,h)
{
}

//Shared display list.
GraphicsWindowFOX::GraphicsWindowFOX(FXComposite* p,FXGLVisual *vis,FXGLCanvas* sharegroup,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h)
: FXGLCanvas(p,vis,sharegroup,tgt,sel,opts,x,y,w,h),
  valid_(false)
{
}

GraphicsWindowFOX::~GraphicsWindowFOX()
{
  valid_ = false;
}

void GraphicsWindowFOX::create()
{
  FXGLCanvas::create();
  valid_ = true;
}

bool GraphicsWindowFOX::realizeImplementation()
{ 
  if(!valid_)
  {
    FXGLCanvas::getVisual()->create();
    FXGLCanvas::create();
    valid_ = true;
  }

  return true;
}

long GraphicsWindowFOX::onConfigure(FXObject *,FXSelector,void *data)
{
  if(target && target->handle(this, MKUINT(message, SEL_CONFIGURE), data)) return 1;

  getEventQueue()->windowResize(0, 0, getWidth(), getHeight());

  return 1;
}

long GraphicsWindowFOX::onMotion(FXObject *,FXSelector,void *data)
{
  if(isEnabled())
  {
    if(target && target->handle(this, MKUINT(message, SEL_MOTION), data)) return 1;

    FXEvent *event = (FXEvent*)data;
    getEventQueue()->mouseMotion(event->win_x, getHeight() - event->win_y);

    return 1;
  }
  return 0;
}

long GraphicsWindowFOX::onMouseWheel(FXObject *,FXSelector,void *data)
{
  if(isEnabled())
  {
    if(target && target->handle(this, MKUINT(message, SEL_MOUSEWHEEL), data)) return 1;
  }
  return 0;
}

long GraphicsWindowFOX::onKeyPress(FXObject *,FXSelector,void *data)
{
  if(isEnabled())
  {
    // Check for applicatio override
    if(target && target->handle(this, MKUINT(message, SEL_KEYPRESS), data)) return 1;

    // This traps all key input
//    FXEvent *event = (FXEvent*)data;
//    getEventQueue()->keyPress( (osgGA::GUIEventAdapter::KeySymbol) event->code );

//    return 1;
  }

  return 0;
}

long GraphicsWindowFOX::onKeyRelease(FXObject *, FXSelector, void *data)
{
  if(isEnabled())
  {
    if(target && target->handle(this, MKUINT(message, SEL_KEYPRESS), data)) return 1;

    FXEvent *event = (FXEvent*)data;
    getEventQueue()->keyRelease( (osgGA::GUIEventAdapter::KeySymbol) event->code );

    return 1;
  }

  return 0;
}

long GraphicsWindowFOX::onLeftBtnPress(FXObject *, FXSelector, void *data)
{
  flags &= ~FLAG_TIP;
  handle(this, MKUINT(0,SEL_FOCUS_SELF), data);
  if(isEnabled())
  {
    grab();
    if(target && target->handle(this, MKUINT(message,SEL_LEFTBUTTONPRESS), data)) return 1;

    FXEvent *event = (FXEvent*)data;
    getEventQueue()->mouseButtonPress(event->win_x, getHeight() - event->win_y, 1);

    return 1;
  }

  return 0;
}

long GraphicsWindowFOX::onLeftBtnRelease(FXObject *, FXSelector, void *data)
{
  if(isEnabled())
  {
    ungrab();
    if(target && target->handle(this, MKUINT(message, SEL_LEFTBUTTONRELEASE), data)) return 1;

    FXEvent *event = (FXEvent*)data;
    getEventQueue()->mouseButtonRelease(event->win_x, getHeight() - event->win_y, 1);

    return 1;
  }

  return 0;
}

long GraphicsWindowFOX::onMiddleBtnPress(FXObject *,FXSelector,void *data)
{
  flags &= ~FLAG_TIP;
  handle(this, MKUINT(0,SEL_FOCUS_SELF), data);
  if(isEnabled())
  {
    grab();
    if(target && target->handle(this, MKUINT(message,SEL_LEFTBUTTONPRESS), data)) return 1;

    FXEvent *event = (FXEvent*)data;
    getEventQueue()->mouseButtonPress(event->win_x, getHeight() - event->win_y, 2);

    return 1;
  }

  return 0;
}

long GraphicsWindowFOX::onMiddleBtnRelease(FXObject *,FXSelector,void *data)
{
  if(isEnabled())
  {
    ungrab();
    if(target && target->handle(this, MKUINT(message, SEL_LEFTBUTTONRELEASE), data)) return 1;

    FXEvent *event = (FXEvent*)data;
    getEventQueue()->mouseButtonRelease(event->win_x, getHeight() - event->win_y, 2);

    return 1;
  }

  return 0;
}

long GraphicsWindowFOX::onRightBtnPress(FXObject *,FXSelector,void *data)
{
  flags &= ~FLAG_TIP;
  handle(this, MKUINT(0,SEL_FOCUS_SELF), data);
  if(isEnabled())
  {
    grab();
    if(target && target->handle(this, MKUINT(message,SEL_LEFTBUTTONPRESS), data)) return 1;

    FXEvent *event = (FXEvent*)data;
    getEventQueue()->mouseButtonPress(event->win_x, getHeight() - event->win_y, 3);

    return 1;
  }

  return 0;
}

long GraphicsWindowFOX::onRightBtnRelease(FXObject *,FXSelector,void *data)
{
  if(isEnabled())
  {
    ungrab();
    if(target && target->handle(this, MKUINT(message, SEL_LEFTBUTTONRELEASE), data)) return 1;

    FXEvent *event = (FXEvent*)data;
    getEventQueue()->mouseButtonRelease(event->win_x, getHeight() - event->win_y, 3);

    return 1;
  }

  return 0;
}
